//+------------------------------------------------------------------+
//|                                                  STO_CCI_RSI.mq4 |                                   
//+------------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 Yellow
#property indicator_color4 Red

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    RsiPer          =  13;
extern int    StochPer        =   8;
extern int    CCIPer          =   8;
extern double Hot             = 0.4;
extern int    Smoothing       =   4;
extern bool   Interpolate     = true;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

double Signal1[],Signal2[],upX[],downX[],trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
	string FileName;
	IndicatorBuffers(5);
      SetIndexBuffer(0,Signal1);
      SetIndexBuffer(1,Signal2);
      SetIndexBuffer(2,upX);   SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,159);
      SetIndexBuffer(3,downX); SetIndexStyle(3,DRAW_ARROW);	SetIndexArrow(3,159);
      SetIndexBuffer(4,trend);

      for (int i=0; i < 4; i++) SetIndexDrawBegin(i,RsiPer+StochPer+CCIPer+Hot+Smoothing);

      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

      //
      //
      //
      //
      //
      
   IndicatorShortName(timeFrameToString(timeFrame)+" STO CCI RSI ("+RsiPer+","+StochPer+","+CCIPer+","+DoubleToStr(Hot,2)+","+Smoothing+")"); 
	return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,n,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { Signal1[0] = limit+1; return(0); }
	
      	double Sm = Smoothing;
         	if (Sm<2) Sm=2;
               double sk  = 2.0 / (Sm + 1.0);
               double sk2 = 2.0 / (Sm * 0.8 + 1.0);

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--)
      {
         double rsi   =  iRSI(NULL,0,RsiPer,PRICE_CLOSE,i);
         double maxrsi = rsi;
         double minrsi = rsi;

                  for (int k=1; k<=StochPer; k++)
                  {
                     double trsi   = iRSI(NULL,0,RsiPer,PRICE_CLOSE,i+k);
                            maxrsi = MathMax(maxrsi,trsi);
                            minrsi = MathMin(minrsi,trsi);
                  }
                  if (maxrsi==minrsi)
         		       double storsi = 0;
                  else  	  storsi = ((rsi-minrsi)/(maxrsi-minrsi)*200-100);
         
         double SCR = Hot*iCCI(NULL,0,CCIPer,PRICE_TYPICAL,i)+(1-Hot)*storsi;

         //
         //
         //
         //
         //
      
   		Signal1[i] = sk *SCR         +(1-sk )*Signal1[i+1];
         Signal2[i] = sk2*Signal1[i+1]+(1-sk2)*Signal2[i+1];
         upX[i]     = EMPTY_VALUE;
         downX[i]   = EMPTY_VALUE;
         trend[i]   = trend[i+1];
		
         if (Signal1[i]>Signal2[i]) trend[i] =  1;
         if (Signal1[i]<Signal2[i]) trend[i] = -1;
         if (trend[i] != trend[i+1])
               if (trend[i]==1) 
                     upX[i]   = Signal2[i]-15;
               else  downX[i] = Signal2[i]+15;
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Signal1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPer,StochPer,CCIPer,Hot,Smoothing,0,y);
         Signal2[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPer,StochPer,CCIPer,Hot,Smoothing,1,y);
         trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPer,StochPer,CCIPer,Hot,Smoothing,4,y);
         upX[i]     = EMPTY_VALUE;
         downX[i]   = EMPTY_VALUE;

            //
            //
            //
            //
            //
            
            if (trend[i]!=trend[i+1])
                  if (trend[i]==1) 
                        upX[i]   = Signal2[i]-15;
                  else  downX[i] = Signal2[i]+15;
            if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

            //
            //
            //
            //
            //

            datetime time = iTime(NULL,timeFrame,y);
               for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(k = 1; k < n; k++)
               {
                  Signal1[i+k] = Signal1[i] + (Signal1[i+n] - Signal1[i])*k/n;
                  Signal2[i+k] = Signal2[i] + (Signal2[i+n] - Signal2[i])*k/n;
               }                  
   }  
   
   //
   //
   //
   //
   //
   
   manageAlerts();
   return(0);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," STO CCI Rsi - at ",TimeToStr(TimeLocal(),TIME_SECONDS)," trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Sto CCI Rsi"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}